# g\x92\xab\xb6b\xba\xc5\xb0\x80\xe3=T9\x8d!i\r\xdcu\x9d\xa8\x02q\x98\xac\xf0\xa8\x99Z\xf1C\x9fV\xf3\xf4e\xda\x18\xd50wC.\x8e\x94\x14-\x10
# Name:    Unit Converter
# Author:  Joseph Nicholas
# Version: 2.3
#
# Copyright (c) 2004-2005 Joseph Nicholas
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
#
#                VERSION HISTORY
#
# Changes from v2.2 to v2.3:
#   -You can now add and delete custom units
#   -Length: microns, nanometers, astronomical units
#   -Pressure: millibars
#   -Mass/Weight: troy ounces, troy pounds
#
# Changes from v2.1 to v2.2:
#   -Added an option to display radians in terms of pi
#   -Made the program always perform floating point division
#   -Circular Measure: deg-min-sec
#
# Changes from v2.0 to v2.1:
#   -Support for mathematical operations and constants (pi) in the input box
#   -Moved Settings and About buttons to menu bar
#   -Automatic restart after settings are saved
#   -Added a help button
#   -New Category: Power
#   -New Category: Metric Prefixes
#   -New Category: Number Names (example: billion, trillion, etc...)
#   -Energy/Work: megajoules, ergs, British thermal units, electron volts
#   -Volume: gallons (UK), quarts (UK), pints (UK), fluid ounces (UK)
#
# Changes from v1.2 to v2.0:
#   -Completely redid the user interface (changed from Tk to wxWidgets)
#   -Settings window
#   -Ability to change the order the units are sorted in
#   -Significant figures instead of decimal places
#   -Ability to change the default number of significant figures
#   -Fixed some errors in the conversion factors
#   -Changed number of lines in list
#   -Changed significant figures to a SpinCtrl instead of TextCtrl
#   -Speed/Velocity: knots, speed of light in a vacuum
#   -Length: fathoms, leagues
#   -Energy/Work: kilocalories
#   -Pressure: pascals, kilopascals
#   -New Category: Circular Measure
#
# Changes from v1.1 to v1.2:
#   -Speed/Velocity: kilmeters per minute, kilometers per second, miles per minute, miles per second
#
# Changes from v1.0 to v1.1:
#   -Added decimal places
#   -Speed/Velocity: mach number
#   -Length: light years, parsecs
#   -Time: months, years
#   -New Category: Area

from __future__ import division
import sys, wx, wx.html, os.path, fraction
from math import log10,floor,pi

notebookpages={0:"length",1:"mass",2:"speed",3:"volume",4:"energy",5:"area",6:"pressure",7:"temp",8:"time",9:"circular",10:"mprefix",11:"numnames",12:"power"}
shortcategorynames=["length","mass","speed","volume","energy","area","pressure","time","circular","mprefix","numnames","power"] # Temp excluded
longcategorynames=['Length','Mass/Weight','Speed/Velocity','Volume','Energy/Work','Area','Pressure','Time',"Circular Measure","Metric Prefixes","Number Names","Power"] # Temperature excluded
fullcategorynames=['Length','Mass/Weight','Speed/Velocity','Volume','Energy/Work','Area','Pressure','Temperature','Time',"Circular Measure","Metric Prefixes","Number Names","Power"]

def export_conversion_factors(filename=os.path.join("data","conversion_factors.ini")):
    final_string = "# CONVERSION FACTORS (temperature separate because it is equations)\n\n"
    for catind in range(12):
        category = shortcategorynames[catind]
        fullcatname = longcategorynames[catind]
        catcf = category + "cf"
        final_string += '# ' + fullcatname + '\n'
        final_string += catcf + "={\\\n"
        for key in sort(eval(catcf)):
            value = eval(category + 'cf["' + key + '"]')
            final_string += '    "' + key + '":' + str(value) + ',\n'
        final_string = final_string[:-2]
        final_string = final_string + "}\n\n"
    final_string = final_string[:-2]
    output_file = file(filename,'w')
    output_file.write(final_string)
    output_file.close()
    return final_string

# Load conversion factors
try:
    conv_facts_file = file(os.path.join("data","conversion_factors.ini"),'r')
    conv_facts = conv_facts_file.read()
    conv_facts_file.close()
    for category in shortcategorynames:
        if category not in conv_facts:
            raise Error("Not all categories were in the file")
    if 'from os import system' not in conv_facts and 'os.system' not in conv_facts: exec(conv_facts)
    else: raise Error("Dangerous things in conversion_factors.ini (eg os.system)")
except:
    def_conv_facts_file = file(os.path.join("data","default_conversion_factors"),'r')
    def_conv_facts = def_conv_facts_file.read()
    def_conv_facts_file.close()
    if 'from os import system' not in def_conv_facts and 'os.system' not in def_conv_facts: exec(def_conv_facts)
    else: raise Error("Dangerous things in def_conversion_factors (eg os.system)")

# Default settings in case there is a problem with the settings file
sortorder="alph"
reverse=False
sigfigdefault="4"
numrows=12 # number of rows in the list boxes

windowxsize=632

def refreshsettings():
    global sortorder, reverse, sigfigdefault, numrows
    try: # Read the configuration file settings
        configfile=open(os.path.join("data","UCsettings.ini"),'r')
        config=configfile.readlines()
        configfile.close()
        for line in config:
            if line[-1]=="\n": line=line[:-1]
            if line=="sort order=by magnitude": sortorder="mag"
            if line=="sort order=alphabetically": sortorder="alph"
            if line=="reverse order=yes": reverse=True
            if line=="reverse order=no": reverse=False
            
            if line[:28]=="default significant figures=": sigfigdefault=line[28:]
            
            if line[:30]=="number of units on one screen=": numrows=int(line[30:])
            
    except: print "Could not load configuration file, using defaults" # If configuration file doesn't exist, leave the defaults

refreshsettings()
    
def sort(dictionary):
    if sortorder=="alph":
        keys=dictionary.keys()
        keys.sort()
    if sortorder=="mag":
        values=dictionary.values()
        values.sort()
        keys=[]
        for value in values:
            for pair in dictionary.items():
                if value==pair[1]: 
                    if pair[0] not in keys: keys.append(pair[0])
        keys.reverse()
    if reverse==True: keys.reverse()
    return keys

def sigfig(x,n):
    if x==0: return x
    if n<0:
        n=sigfig(-n,1)
        return n*int(x/n+.5)
    if n==0: return 10.**(int(floor(log10(abs(x)))))
    return round(x,int(n)-1-int(floor(log10(abs(x)))))

def deg2deg_min_sec(degrees, mode='string'):
    "Converts degrees to a string/list/tuple containing degrees, minutes, and seconds"
    tsecs=degrees*3600
    deg=int(tsecs//3600)
    min=int((tsecs//60)-((tsecs//3600)*60))
    sec=int(tsecs-((tsecs//60)-((tsecs//3600)*60))*60-((tsecs//3600)*3600))
    if mode=='list': return [deg,min,sec]
    elif mode=='tuple': return (deg,min,sec)
    else: return str(deg)+chr(176)+str(min)+"'"+str(sec)+'"'

def interms(value, interm, interm_str):
    frac=fraction.Fraction(value/interm)
    str_frac=str(frac)
    str_frac='('+str_frac
    try: 
        str_frac=str_frac[:str_frac.index('/')]+"*"+interm_str+') / '+str_frac[str_frac.index('/')+1:]
        if "(1*" in str_frac:
            str_frac=str_frac.replace("(1*","")
            str_frac=str_frac.replace(")","")
    except ValueError:
        if frac==1: return interm_str
        else: str_frac=str_frac[1:]+"*"+interm_str
    if eval(str_frac)==0: return "0"
    return str_frac

def create(parent):
    return MainFrame(parent)
    
categories=notebookpages.values()
shift=windowxsize/2-271

class MainFrame(wx.Frame):
    def _init_coll_notebook1_Pages(self, parent):
        """
        sortfullcat=list(fullcategorynames)
        sortfullcat.sort()
        sortcat=list(categories)
        sortcat.sort()
        cat_ind=0
        while cat_ind<len(categories):
            temp="parent.AddPage(imageId=-1, page=self.CapCatWindow, select=False, text='Category')".replace('Category',sortfullcat[cat_ind])
            exec(temp.replace("CapCat", sortcat[cat_ind].capitalize()))
            cat_ind+=1
        """
        
        parent.AddPage(imageId=-1, page=self.LengthWindow, select=True,
              text='Length')
        parent.AddPage(imageId=-1, page=self.MassWindow, select=False,
              text='Mass/Weight')
        parent.AddPage(imageId=-1, page=self.SpeedWindow, select=False,
              text='Speed/Velocity')
        parent.AddPage(imageId=-1, page=self.VolumeWindow, select=False,
              text='Volume')
        parent.AddPage(imageId=-1, page=self.EnergyWindow, select=False,
              text='Energy/Work')
        parent.AddPage(imageId=-1, page=self.AreaWindow, select=False,
              text='Area')
        parent.AddPage(imageId=-1, page=self.PressureWindow, select=False,
              text='Pressure')
        parent.AddPage(imageId=-1, page=self.TempWindow, select=False,
              text='Temperature')
        parent.AddPage(imageId=-1, page=self.TimeWindow, select=False,
              text='Time')
        parent.AddPage(imageId=-1, page=self.CircularWindow, select=False,
              text='Circular Measure')
        parent.AddPage(imageId=-1, page=self.MprefixWindow, select=False,
              text='Metric Prefixes')
        parent.AddPage(imageId=-1, page=self.NumnamesWindow, select=False,
              text='Number Names')
        parent.AddPage(imageId=-1, page=self.PowerWindow, select=False,
              text='Power')
    
    def _init_ctrls(self, prnt):
        self.numrows=numrows-10
        wx.Frame.__init__(self, id=-1, name='', parent=prnt,
              pos=wx.Point(364, 345), size=wx.Size(windowxsize, 376+13.5*self.numrows),
              style=wx.DEFAULT_FRAME_STYLE, title='Unit Converter')
        self.SetClientSize(wx.Size(windowxsize, 360+13.5*self.numrows))
        self.SetIcon(wx.Icon(os.path.join("data","UCicon.ico"), wx.BITMAP_TYPE_ICO))
        self.Bind(wx.EVT_CLOSE, self.OnMainClose)
        self.Center(wx.BOTH)
        
        try:    self.notebook1 = wx.Notebook(id=-1, name='notebook1', parent=self, pos=wx.Point(0, 0), size=self.GetSize(), style=wx.MAXIMIZE_BOX | wx.NO_BORDER | wx.NB_MULTILINE)
        except: self.notebook1 = wx.Notebook(id=-1, name='notebook1', parent=self, pos=wx.Point(0, 0), size=self.GetSize(), style=wx.MAXIMIZE_BOX | wx.NO_BORDER)
        
        
        for cat in categories: exec("self.LengthWindow = wx.Panel(id=-1, name='LengthWindow', parent=self.notebook1, pos=wx.Point(0, 0), style=wx.MAXIMIZE_BOX)".replace("Length",cat.capitalize()))
        self._init_coll_notebook1_Pages(self.notebook1)
        for cat in categories: exec("self.LengthWindow.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_BTNFACE))".replace("Length",cat.capitalize()))
        
        self.strcreatewidgets="""

self.LengthOutput = wx.ListBox(choices=sort(lengthcf), id=-1, name='LengthOutput', parent=self.LengthWindow, pos=wx.Point(280+shift, 32), size=wx.Size(200, %f), style=0)
self.LengthOutput.Bind(wx.EVT_LISTBOX, self.calcconversion, id=-1)

self.LengthInput = wx.ListBox(choices=sort(lengthcf), id=-1, name='LengthInput', parent=self.LengthWindow, pos=wx.Point(64+shift, 32), size=wx.Size(200, %f), style=0)
self.LengthInput.Bind(wx.EVT_LISTBOX, self.calcconversion, id=-1)

self.LengthInputValue = wx.TextCtrl(id=-1, name='LengthInputValue', parent=self.LengthWindow, pos=wx.Point(176+shift, %f), size=wx.Size(192, 21), style=wx.TE_PROCESS_ENTER | wx.TE_CENTER, value='0')
self.LengthInputValue.Bind(wx.EVT_TEXT, self.calcconversion, id=-1)
self.LengthInputValue.Bind(wx.EVT_TEXT_ENTER, self.calcconversion, id=-1)

self.LengthOutputValue = wx.TextCtrl(id=-1, name='LengthOutputValue', parent=self.LengthWindow, pos=wx.Point(176+shift, %f), size=wx.Size(192, 21), style=wx.TE_CENTER | wx.TE_READONLY, value='0')

self.LengthDecimalPlaces = self.sigfigspinctrl = wx.SpinCtrl(id=-1, value=sigfigdefault, max=10, min=1, name='LengthDecimalPlaces', parent=self.LengthWindow, pos=wx.Point(304+shift, %f), size=wx.Size(48, 21), style=wx.SP_ARROW_KEYS)
self.LengthDecimalPlaces.Bind(wx.EVT_TEXT, self.calcconversion, id=-1)
self.LengthDecimalPlaces.Bind(wx.EVT_SPINCTRL, self.calcconversion, id=-1)

self.ivlengthlabel = wx.StaticText(id=-1, label='Input Value:', name='ivlengthlabel', parent=self.LengthWindow, pos=wx.Point(104+shift, %f), size=wx.Size(69, 16), style=0)
self.ivlengthlabel.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.NORMAL, False, 'Microsoft Sans Serif'))

self.ovlengthlabel = wx.StaticText(id=-1, label='Output Value:', name='ovlengthlabel', parent=self.LengthWindow, pos=wx.Point(96+shift, %f), size=wx.Size(79, 16), style=0)
self.ovlengthlabel.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.NORMAL, False, 'Microsoft Sans Serif'))

self.LengthInputUnitText = wx.TextCtrl(id=-1, name='LengthInputUnitText', parent=self.LengthWindow, pos=wx.Point(376+shift, %f), size=wx.Size(152, 21), style=wx.TE_READONLY | wx.NO_BORDER | wx.TRANSPARENT_WINDOW, value='')
self.LengthInputUnitText.SetBackgroundColour(self.LengthWindow.GetBackgroundColour())
self.LengthInputUnitText.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.NORMAL, False, 'Microsoft Sans Serif'))

self.LengthOutputUnitText = wx.TextCtrl(id=-1, name='LengthOutputUnitText', parent=self.LengthWindow, pos=wx.Point(376+shift, %f), size=wx.Size(152, 21), style=wx.NO_BORDER | wx.TRANSPARENT_WINDOW | wx.TE_READONLY, value='')
self.LengthOutputUnitText.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.NORMAL, False, 'Microsoft Sans Serif'))
self.LengthOutputUnitText.SetBackgroundColour(self.LengthWindow.GetBackgroundColour())

self.iulengthlabel = wx.StaticText(id=-1, label='Input Unit', name='iulengthlabel', parent=self.LengthWindow, pos=wx.Point(131+shift, 8), size=wx.Size(70, 20), style=0)
self.iulengthlabel.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL, True, 'Microsoft Sans Serif'))

self.oulengthlabel = wx.StaticText(id=-1, label='Output Unit', name='oulengthlabel', parent=self.LengthWindow, pos=wx.Point(338+shift, 8), size=wx.Size(82, 20), style=0)
self.oulengthlabel.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL, True, 'Microsoft Sans Serif'))

self.lengthdp = wx.StaticText(id=-1, label='Significant Figures:', name='lengthdp', parent=self.LengthWindow, pos=wx.Point(190+shift, %f), size=wx.Size(98, 16), style=0)
self.lengthdp.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.NORMAL, False, 'Microsoft Sans Serif'))

""" % (135+self.numrows*13.5,135+self.numrows*13.5,192+self.numrows*13.5,264+self.numrows*13.5,216+self.numrows*13.5,192+self.numrows*13.5,264+self.numrows*13.5,192+self.numrows*13.5,264+self.numrows*13.5, 216+self.numrows*13.5)

        
        # Execute the above string to create widgets for all the notebook pages
        for cat in categories:
            self.createwidgets=self.strcreatewidgets.replace("length",cat)
            self.createwidgets=self.createwidgets.replace("Length",cat.capitalize())
            if cat=="temp":
                self.createwidgets=self.createwidgets.replace("choices=sort(tempcf)",'choices=["celsius","fahrenheit","kelvin"]')
                self.createwidgets=self.createwidgets.replace("self.calcconversion","self.calctemp")
            exec(self.createwidgets)
        
        # Add the "In terms of pi" checkbox to the circular measure window
        self.picheckbox = wx.CheckBox(id=-1,
              label='Radians in terms of pi', name='picheckbox', parent=self.CircularWindow,
              pos=wx.Point(260, 270), size=wx.Size(120, 13), style=0)
        self.picheckbox.SetValue(False)
        self.picheckbox.Bind(wx.EVT_CHECKBOX, self.calcconversion, id=-1)
        self.picheckbox.Enable(False)
        
        # Create the menubar at the top of the frame
        [ID_SETTINGS, ID_ADD_UNIT, ID_DELETE_UNIT, ID_RESTORE_DEFAULTS, ID_HELP,
          ID_ABOUT] = [wx.NewId() for IDs in range(6)]
        menubar = wx.MenuBar()
        settingsmenu = wx.Menu()
        settingsmenu.Append(ID_SETTINGS, "Change Settings")
        unitsmenu = wx.Menu()
        unitsmenu.Append(ID_ADD_UNIT, "Add New Unit")
        unitsmenu.Append(ID_DELETE_UNIT, "Delete Unit")
        unitsmenu.AppendSeparator()
        unitsmenu.Append(ID_RESTORE_DEFAULTS, "Restore Default Units")
        helpmenu = wx.Menu()
        helpmenu.Append(ID_HELP, "Help")
        helpmenu.Append(ID_ABOUT, "About")
        menubar.Append(settingsmenu, "Settings")
        menubar.Append(unitsmenu, "Units")
        menubar.Append(helpmenu, "Help")
        self.SetMenuBar(menubar)
        self.Bind(wx.EVT_MENU, self.opensettings, id=ID_SETTINGS)
        self.Bind(wx.EVT_MENU, self.open_add_dlg, id=ID_ADD_UNIT)
        self.Bind(wx.EVT_MENU, self.open_delete_dlg, id=ID_DELETE_UNIT)
        self.Bind(wx.EVT_MENU, self.restore_defaults, id=ID_RESTORE_DEFAULTS)
        self.Bind(wx.EVT_MENU, self.showhelp, id=ID_HELP)
        self.Bind(wx.EVT_MENU, self.aboutmsgfunc, id=ID_ABOUT)
    
    
    def __init__(self, parent): self._init_ctrls(parent)
    
    def restore_defaults(self, event):
        confirmdlg=wx.MessageDialog(self, "Are you sure you want to restore all default units?", caption="Restore Default Units?", style=wx.YES_NO)
        choice = confirmdlg.ShowModal()
        if choice == wx.ID_YES:
            filename = os.path.join("data", "conversion_factors.ini")
            clear_file = file(filename, 'w')
            clear_file.close()
            
            def_conv_facts_file = file(os.path.join("data","default_conversion_factors"),'r')
            def_conv_facts = def_conv_facts_file.read()
            def_conv_facts_file.close()
            if 'from os import system' not in def_conv_facts and 'os.system' not in def_conv_facts: exec(def_conv_facts)
            else: raise Error("Dangerous things in def_conversion_factors (eg os.system)")
            
            for category in shortcategorynames:
                exec('self.' + category.capitalize() + 'Input.Clear()')
                exec('self.' + category.capitalize() + 'Output.Clear()')
                for item in sort(eval(category + 'cf')):
                    exec('self.' + category.capitalize() + 'Input.Append(item)')
                for item in sort(eval(category + 'cf')):
                    exec('self.' + category.capitalize() + 'Output.Append(item)')
    
    def open_add_dlg(self, event):
        add_unit_dlg = add_unit_dialog(self)
        add_unit_dlg.ShowModal()
    
    def open_delete_dlg(self, event):
        del_unit_dlg = delete_unit_dialog(self)
        del_unit_dlg.ShowModal()
    
    def opensettings(self,event):
        settingsdia=Settings(self, 'Settings','Settings')
        settingsdia.ShowModal()
    
    def showhelp(self,event):
        helpmsg=HTMLdialog(self, "Help", os.path.join("data","Help.html"), size=wx.Size(550,400), showOK=False)
        helpmsg.ShowModal()
    
    def OnMainClose(self,event): sys.exit()
    
    def aboutmsgfunc(self,event):
        aboutmsg=HTMLdialog(self, "About", os.path.join("data","About.html"), size=wx.Size(400,300), buttontext="Close")
        aboutmsg.ShowModal()
    
    
    def calcconversion(self,event):
        err=False
        cat=self.notebook1.GetSelection()
        cat=notebookpages[cat]
        exec("""try: self.round2=int(self.%sDecimalPlaces.GetValue())
except ValueError: self.%sOutputValue.SetValue("Invalid significant figures")""" % (cat.capitalize(),cat.capitalize()))
        if self.round2 not in range(1,11):
            exec('self.%sOutputValue.SetValue("Significant figures must be 1-10")' % cat.capitalize())
            return 0
        
        self.strcalcfunc="""
self.curinputlength=self.LengthInput.GetStringSelection()
self.curoutputlength=self.LengthOutput.GetStringSelection()
try: self.curuserinput=float(eval(self.LengthInputValue.GetValue().replace('^','**')))
except: pass
self.LengthInputUnitText.SetValue(self.LengthInput.GetStringSelection())
self.LengthOutputUnitText.SetValue(self.LengthOutput.GetStringSelection())

# convert deg-min-sec to degrees if necessary
try:
    if cat=="circular" and self.CircularInput.GetStringSelection()=='deg-min-sec':
        temp=eval(self.CircularInputValue.GetValue())
        self.curuserinput=temp[0]+temp[1]/60.0+temp[2]/3600.0
except (TypeError,IndexError):
    self.CircularOutputValue.SetValue("separate w/ commas. ex deg,min,sec")
    err=True

try:    
    self.temporary=self.curuserinput/lengthcf[self.curinputlength]
    self.final=self.temporary*lengthcf[self.curoutputlength]
except KeyError: err=True"""

        self.calcfunc=self.strcalcfunc.replace("length",cat)
        self.calcfunc=self.calcfunc.replace("Length",cat.capitalize())
        exec(self.calcfunc)
        
        if not err:
            
            # Make the result in terms of pi if the output is radians and the box is checked
            if cat=="circular" and self.picheckbox.IsChecked()==True and self.CircularOutput.GetStringSelection()=='radians':
                self.CircularOutputValue.SetValue(interms(self.final, pi, 'pi'))
            
            # For the deg-min-sec unit
            elif cat=="circular" and self.CircularOutput.GetStringSelection()=='deg-min-sec':
                self.CircularOutputValue.SetValue(deg2deg_min_sec(self.final))
            
            else: # all other units
                exec("self.LengthOutputValue.SetValue(str(sigfig(self.final, self.round2)))".replace("Length",cat.capitalize()))
        
        
        # Enable the "Radians in terms of pi" checkbox if the conditions are right. Disable otherwise.
        if cat=="circular" and self.CircularOutput.GetStringSelection()=='radians':
            self.picheckbox.Enable(True)
        else: self.picheckbox.Enable(False)
        
    
    def calctemp(self,event):
        self.curinputtemp=self.TempInput.GetStringSelection()
        self.curoutputtemp=self.TempOutput.GetStringSelection()
        try: self.curuserinput=float(eval(self.TempInputValue.GetValue().replace('^','**')))
        except: pass
        try: self.round2=int(self.TempDecimalPlaces.GetValue())
        except ValueError: self.TempOutputValue.SetValue("Invalid significant figures")
        self.TempInputUnitText.SetValue(self.TempInput.GetStringSelection())
        self.TempOutputUnitText.SetValue(self.TempOutput.GetStringSelection())
        
        if self.curinputtemp=="fahrenheit":
            if self.curoutputtemp=="fahrenheit":
                self.final=self.curuserinput
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="fahrenheit":
            if self.curoutputtemp=="celsius":
                self.final=(self.curuserinput-32)*5/9
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="fahrenheit":
            if self.curoutputtemp=="kelvin":
                self.a=(self.curuserinput-32)*5/9
                self.final=self.a+273
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="celsius":
            if self.curoutputtemp=="fahrenheit":
                self.final=((self.curuserinput*9)/5)+32
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="celsius":
            if self.curoutputtemp=="celsius":
                self.final=self.curuserinput
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="celsius":
            if self.curoutputtemp=="kelvin":
                self.final=self.curuserinput+273
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="kelvin":
            if self.curoutputtemp=="fahrenheit":
                self.a=self.curuserinput-273
                self.final=((self.a*9)/5)+32
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="kelvin":
            if self.curoutputtemp=="celsius":
                self.final=self.curuserinput-273
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))
        
        if self.curinputtemp=="kelvin":
            if self.curoutputtemp=="kelvin":
                self.final=self.curuserinput
                self.TempOutputValue.SetValue(str(sigfig(self.final, self.round2)))

class Settings(wx.Dialog):
    def __init__(self, parent, msg, caption, pos=wx.DefaultPosition, size=wx.Size(314,228), style=wx.DEFAULT_DIALOG_STYLE):
        wx.Dialog.__init__(self, parent, -1, caption, pos, size, style)
        self.SetClientSize(wx.Size(306, 194))
        self.Center(wx.BOTH)
        
        self.settingstitle = wx.StaticText(id=-1, label='Settings',
              name='settingstitle', parent=self, pos=wx.Point(120, 0),
              size=wx.Size(66, 24), style=wx.ALIGN_CENTRE)
        self.settingstitle.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.NORMAL, True,
              'Microsoft Sans Serif'))
        self.settingstitle.Center(wx.HORIZONTAL)
        
        self.unitsorderbox = wx.StaticBox(id=-1, label='List Units:',
              name='unitsorderbox', parent=self, pos=wx.Point(8, 32),
              size=wx.Size(128, 80), style=0)
        
        self.orderchoice = wx.Choice(choices=["alphabetically", "by magnitude"],
              id=-1, name='orderchoice', parent=self, pos=wx.Point(24, 56),
              size=wx.Size(96, 21), style=0)
        self.orderchoice.SetSelection(-1)
        
        self.reversecheckbox = wx.CheckBox(id=-1, label='Reverse',
              name='reversecheckbox', parent=self, pos=wx.Point(32, 88),
              size=wx.Size(73, 13), style=0)
        self.reversecheckbox.SetValue(False)
        
        self.savesettingsbutton = wx.Button(id=-1,
              label='Save Settings and Close', name='savesettingsbutton',
              parent=self, pos=wx.Point(88, 160), size=wx.Size(128, 23),
              style=0)
        self.savesettingsbutton.Bind(wx.EVT_BUTTON, self.savesettings, id=-1)
        
        self.notetext1 = wx.StaticText(id=-1,
              label='Unit Converter will restart when settings are saved',
              name='notetext', parent=self, pos=wx.Point(40, 136),
              size=wx.Size(300, 13), style=0)
        
        self.sigfigbox = wx.StaticBox(id=-1,
              label='Default Significant Figures:', name='sigfigbox',
              parent=self, pos=wx.Point(144, 32), size=wx.Size(144, 48),
              style=0)
        
        self.sigfigspinctrl = wx.SpinCtrl(id=-1, initial=0, max=10, min=1,
              name='sigfigspinctrl', parent=self, pos=wx.Point(193, 52),
              size=wx.Size(48, 21), style=wx.SP_ARROW_KEYS)
        
        self.staticBox1 = wx.StaticBox(id=-1, label='# of units on one screen:',
              name='staticBox1', parent=self, pos=wx.Point(144, 88),
              size=wx.Size(144, 40), style=0)
        
        self.numlinesctrl = wx.SpinCtrl(id=-1, initial=0, max=100, min=1,
              name='numlinesctrl', parent=self, pos=wx.Point(192, 104),
              size=wx.Size(48, 21), style=wx.SP_ARROW_KEYS)
        
        refreshsettings()
        
        if sortorder=="alph": self.orderchoice.SetSelection(0) # set to alphabetically
        if sortorder=="mag": self.orderchoice.SetSelection(1) # set to by magnitude
        
        if reverse==True: self.reversecheckbox.SetValue(True)
        
        self.sigfigspinctrl.SetValue(int(sigfigdefault))
        
        self.numlinesctrl.SetValue(numrows)
        

    def savesettings(self, event):
        global mainapp
        
        configfile=open(os.path.join("data","UCsettings.ini"),"w")
        
        if self.orderchoice.GetLabel()=="by magnitude": configfile.write("sort order=by magnitude\n")
        else: configfile.write("sort order=alphabetically\n")
        
        if self.reversecheckbox.GetValue()==True: configfile.write("reverse order=yes\n")
        else: configfile.write("reverse order=no\n")
        
        sigfigstr="default significant figures="+str(self.sigfigspinctrl.GetValue())+"\n"
        configfile.write(sigfigstr)
        
        numrowsstr="number of units on one screen="+str(self.numlinesctrl.GetValue())+"\n"
        configfile.write(numrowsstr)
        
        configfile.close()
        refreshsettings()
        mainapp.frame.Destroy()
        self.Destroy()
        mainapp=MainApp(0)
        mainapp.MainLoop()

class add_unit_dialog(wx.Dialog):
    def _init_ctrls(self, prnt):
        wx.Dialog.__init__(self, id=-1, name='add_unit_dialog', parent=prnt,
              pos=wx.Point(506, 415), size=wx.Size(242, 215),
              style=wx.DEFAULT_DIALOG_STYLE, title='Add New Unit')
        self.SetClientSize(wx.Size(234, 183))
        self.Center(wx.BOTH)
        
        slcn=list(longcategorynames)
        slcn.sort()
        self.category_choice = wx.Choice(choices=slcn, id=-1,
              name='category_choice', parent=self, pos=wx.Point(56, 2),
              size=wx.Size(130, 21), style=0)
        self.category_choice.Bind(wx.EVT_CHOICE, self.category_chosen, id=-1)
        
        self.orig_value = wx.TextCtrl(id=-1, name='orig_value', parent=self,
              pos=wx.Point(8, 48), size=wx.Size(40, 21), style=wx.TE_CENTER,
              value='')
        self.orig_value.Bind(wx.EVT_TEXT, self.updated, id=-1)
        
        self.orig_unit = wx.Choice(choices=[],
              id=-1, name='orig_unit',
              parent=self, pos=wx.Point(96, 48), size=wx.Size(130, 21),
              style=0)
        self.orig_unit.Bind(wx.EVT_CHOICE, self.updated, id=-1)
        
        self.new_value = wx.TextCtrl(id=-1, name='new_value', parent=self,
              pos=wx.Point(8, 96), size=wx.Size(40, 21), style=wx.TE_CENTER,
              value='')
        self.new_value.Bind(wx.EVT_TEXT, self.updated, id=-1)
        
        self.new_unit = wx.TextCtrl(id=-1, name='new_unit', parent=self,
              pos=wx.Point(96, 96), size=wx.Size(128, 21), style=wx.TE_CENTER,
              value='')
        self.new_unit.Bind(wx.EVT_TEXT, self.updated, id=-1)
        
        self.new_unit_button = wx.Button(id=-1, label='Add New Unit',
              name='new_unit_button', parent=self, pos=wx.Point(56, 152),
              size=wx.Size(136, 23), style=0)
        self.new_unit_button.Bind(wx.EVT_BUTTON, self.new_unit_func, id=-1)
        self.new_unit_button.Enable(False)
        
        self.old_value_st = wx.StaticText(id=-1,
              label='Value', name='old_value_st', parent=self, pos=wx.Point(16,
              32), size=wx.Size(26, 13), style=0)
        
        self.old_unit_st = wx.StaticText(id=-1,
              label='Old Unit', name='old_unit_st', parent=self,
              pos=wx.Point(136, 32), size=wx.Size(38, 13), style=0)
        
        self.new_value_st = wx.StaticText(id=-1,
              label='Value', name='new_value_st', parent=self, pos=wx.Point(16,
              120), size=wx.Size(26, 13), style=0)
        
        self.new_unit_st = wx.StaticText(id=-1,
              label='New Unit Name', name='new_unit_st', parent=self,
              pos=wx.Point(128, 120), size=wx.Size(73, 13), style=0)
        
        self.equals_text = wx.StaticText(id=-1,
              label='equals', name='equals_text', parent=self,
              pos=wx.Point(80, 75), size=wx.Size(54, 13), style=0)
    
    def __init__(self, parent):
        self._init_ctrls(parent)
    
    def category_chosen(self, event):
        ind = longcategorynames.index(self.category_choice.GetStringSelection())
        self.category = shortcategorynames[ind] + "cf"
        self.orig_unit.Clear()
        for item in sort(eval(self.category)): self.orig_unit.Append(item)
        self.orig_unit.SetSelection(-1)
        self.updated(None)
    
    def updated(self, event):
        if self.orig_value.GetValue() and self.new_value.GetValue() and \
          self.new_unit.GetValue() and self.orig_unit.GetSelection() != -1:
            self.new_unit_button.Enable(True)
        else: self.new_unit_button.Enable(False)
    
    def new_unit_func(self, event):
        o_unit = self.orig_unit.GetStringSelection()
        o_val = float(eval(self.orig_value.GetValue()))
        n_val = float(eval(self.new_value.GetValue()))
        n_unit = self.new_unit.GetValue()
        a = n_val / o_val
        b = a * eval(self.category)[o_unit]
        eval(self.category)[n_unit] = b
        export_conversion_factors()
        exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Input.Clear()')
        exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Output.Clear()')
        for item in sort(eval(self.category)):
            exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Input.Append(item)')
        for item in sort(eval(self.category)):
            exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Output.Append(item)')
        self.Destroy()

class delete_unit_dialog(wx.Dialog):
    def _init_ctrls(self, prnt):
        wx.Dialog.__init__(self, id=-1,
              name='delete_unit_dialog', parent=prnt, pos=wx.Point(762, 498),
              size=wx.Size(255, 146), style=wx.DEFAULT_DIALOG_STYLE,
              title='Delete Unit')
        self.SetClientSize(wx.Size(247, 114))
        self.Center(wx.BOTH)
        
        slcn=list(longcategorynames)
        slcn.sort()
        self.category_choice = wx.Choice(choices=slcn,
              id=-1, name='category_choice',
              parent=self, pos=wx.Point(72, 16), size=wx.Size(160, 21),
              style=0)
        self.category_choice.Bind(wx.EVT_CHOICE, self.category_chosen, id=-1)
        
        self.unit_choice = wx.Choice(choices=[],
              id=-1, name='unit_choice',
              parent=self, pos=wx.Point(72, 48), size=wx.Size(160, 21),
              style=0)
        self.unit_choice.Bind(wx.EVT_CHOICE, self.unit_chosen, id=-1)
        
        self.category_st = wx.StaticText(id=-1,
              label='Category:', name='category_st', parent=self,
              pos=wx.Point(8, 16), size=wx.Size(53, 14), style=0)
        self.category_st.SetFont(wx.Font(9, wx.SWISS, wx.NORMAL, wx.NORMAL,
              False, 'Tahoma'))
        
        self.unit_st = wx.StaticText(id=-1,
              label='Unit:', name='unit_st', parent=self, pos=wx.Point(32, 48),
              size=wx.Size(26, 14), style=0)
        self.unit_st.SetFont(wx.Font(9, wx.SWISS, wx.NORMAL, wx.NORMAL, False,
              'Tahoma'))
        
        self.delete_button = wx.Button(id=-1,
              label='Delete Selected Unit', name='delete_button', parent=self,
              pos=wx.Point(56, 80), size=wx.Size(139, 23), style=0)
        self.delete_button.Bind(wx.EVT_BUTTON, self.delete_unit,
              id=-1)
        self.delete_button.Enable(False)
    
    def __init__(self, parent):
        self._init_ctrls(parent)
    
    def category_chosen(self, event):
        ind = longcategorynames.index(self.category_choice.GetStringSelection())
        self.category = shortcategorynames[ind] + "cf"
        self.unit_choice.Clear()
        for item in sort(eval(self.category)): self.unit_choice.Append(item)
        self.unit_choice.SetSelection(-1)
        self.delete_button.Enable(False)
    
    def unit_chosen(self, event):
        self.delete_button.Enable(True)
    
    def delete_unit(self, event):
        exec("del " + self.category + "['" + self.unit_choice.GetStringSelection() + "']")
        export_conversion_factors()
        exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Input.Clear()')
        exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Output.Clear()')
        for item in sort(eval(self.category)):
            exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Input.Append(item)')
        for item in sort(eval(self.category)):
            exec('mainapp.frame.' + self.category[:-2].capitalize() + 'Output.Append(item)')
        self.Destroy()

class HTMLdialog(wx.Dialog):
    def __init__(self, parent, title, htmlpage, size=wx.Size(500, 500), showOK=True, buttontext="Ok", bg_colour=wx.Colour(0,0,0)):
        wx.Dialog.__init__(self, id=-1, name='', parent=parent, pos=wx.Point(557,
              237), size=size, style=wx.DEFAULT_DIALOG_STYLE,
              title=title)
        self.SetClientSize(self.GetSize())
        self.SetBackgroundColour(bg_colour)
        self.Center(wx.BOTH)
        
        if showOK: HTMLposysub=35
        else: HTMLposysub=0
        
        self.HTMLwin = wx.html.HtmlWindow(id=-1, name='HelpHTMLwin',
              parent=self, pos=wx.Point(0, 0), size=wx.Size(self.GetClientSizeTuple()[0], self.GetClientSizeTuple()[1]-HTMLposysub),
              style=wx.html.HW_SCROLLBAR_AUTO)
        self.HTMLwin.LoadPage(htmlpage)
        
        if showOK:
            self.OKbutton = wx.Button(id=-1, label=buttontext, name='OKbutton',
                parent=self, pos=wx.Point(self.GetClientSizeTuple()[0]/2-38, self.GetClientSizeTuple()[1]-30),
                size=wx.Size(76, 23), style=0)
            self.OKbutton.Bind(wx.EVT_BUTTON, self.pressOK, id=-1)
        
    def pressOK(self, event):
        self.Destroy()

class MainApp(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        self.frame = MainFrame(None)
        self.SetTopWindow(self.frame)
        self.frame.Show()
        return 1
    
if __name__=='__main__':
    mainapp = MainApp(0)
    mainapp.MainLoop()
